package edu.iris.dmc.service;

import static java.net.HttpURLConnection.HTTP_OK;
import static java.util.logging.Level.WARNING;
import static java.util.logging.Level.SEVERE;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.stream.EventFilter;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import edu.iris.dmc.criteria.Criteria;
import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.OutputLevel;
import edu.iris.dmc.criteria.StationCriteria;
import edu.iris.dmc.event.model.Event;
import edu.iris.dmc.fdsn.station.model.FDSNStationXML;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.RestrictedStatusType;
import edu.iris.dmc.fdsn.station.model.Station;
import edu.iris.dmc.ws.util.StringUtil;

/**
 * Station service is the main class for querying the Station service
 * 
 */
public class StationService extends BaseService {

	private static final Logger logger = Logger.getLogger(StationService.class
			.getName());

	public StationService(String baseUrl, String version,
			String compatabilityVersion, String userAgent) {
		super(version, compatabilityVersion, userAgent);
		this.baseUrl = baseUrl;
	}


	public StationIterator iterate(InputStream is)
			throws FileNotFoundException, XMLStreamException, JAXBException {
		return new StationIterator(is);
	}
	
	public StationIterator iterate(Criteria criteria, OutputLevel level)
			throws NoDataFoundException, CriteriaException, IOException,
			ServiceNotSupportedException {
		if (logger.isLoggable(Level.FINER)) {
			logger.entering(this.getClass().getName(), "fetch(criteria, url)",
					new Object[] { criteria, level });
		}

		StringBuilder paramsString = new StringBuilder(criteria.toUrlParams()
				.get(0));

		if (level == OutputLevel.NETWORK) {
			paramsString.append("&level=network").toString();
		} else if (level == OutputLevel.STATION) {
			paramsString.append("&level=station").toString();
		} else if (level == OutputLevel.CHANNEL) {
			paramsString.append("&level=channel").toString();
		} else if (level == OutputLevel.RESPONSE) {
			paramsString.append("&level=response").toString();
		} else {
			paramsString.append("&level=station").toString();
		}
		return this.iterate(this.baseUrl + "query?" + paramsString.toString());

	}

	public StationIterator iterate(String url) throws NoDataFoundException,
			CriteriaException, IOException, ServiceNotSupportedException {
		if (logger.isLoggable(Level.FINER)) {
			logger.entering(this.getClass().getName(), "fetch(String url)",
					new Object[] { url });
		}

		URL theQuery = null;
		try {
			theQuery = new URL(url);

		} catch (MalformedURLException e) {
			throw new IOException("Invalid url: " + url);
		}

		HttpURLConnection connection = null;

		try {
			connection = getConnection(theQuery);
			connection.setRequestMethod("GET");

			String uAgent = this.userAgent;
			if (this.getAppName() != null && !"".equals(this.appName)) {
				uAgent = uAgent + " (" + this.appName + ")";
			}
			connection.setRequestProperty("User-Agent", uAgent);

			connection.setRequestProperty("Accept", "application/xml");
			connection.setRequestProperty("Accept-Encoding", "gzip, deflate");
			connection.connect();

			int responseCode = connection.getResponseCode();
			InputStream inputStream = responseCode != HTTP_OK ? connection
					.getErrorStream() : connection.getInputStream();
			if ("gzip".equals(connection.getContentEncoding())) {
				inputStream = new GZIPInputStream(inputStream);
			}
			switch (responseCode) {
			case 404:
				if (logger.isLoggable(WARNING))
					logger.warning("No data Found for the GET request "
							+ theQuery + StringUtil.toString(inputStream));
				return null;
			case 204:
				if (logger.isLoggable(WARNING))
					logger.warning("No data Found for the GET request "
							+ theQuery);
				throw new NoDataFoundException("No data found for: " + theQuery);
			case 400:
				if (logger.isLoggable(SEVERE))
					logger.severe("An error occurred while making a GET request "
							+ theQuery + StringUtil.toString(inputStream));
				throw new CriteriaException("Bad request parameter: "
						+ StringUtil.toString(inputStream));
			case 500:
				if (logger.isLoggable(WARNING))
					logger.severe("An error occurred while making a GET request "
							+ theQuery + StringUtil.toString(inputStream));
				throw new IOException(StringUtil.toString(inputStream));
			case 200:
				return new StationIterator(connection, inputStream);
			default:
				throw new IOException(connection.getResponseMessage());
			}

		} catch (JAXBException e) {
			throw new IOException(e);
		} catch (XMLStreamException e) {
			e.printStackTrace();
			throw new IOException(e);
		}
	}

	/**
	 * Load Networks from inputstream, example local file
	 * 
	 * @param inputStream
	 * @return the networks list from the file
	 * @throws IOException
	 */
	public List<Network> load(InputStream inputStream) throws IOException {

		try {
			JAXBContext jc = JAXBContext
					.newInstance("edu.iris.dmc.fdsn.station.model");
			Unmarshaller unmarshaller = jc.createUnmarshaller();
			unmarshaller
					.setEventHandler(new javax.xml.bind.helpers.DefaultValidationEventHandler());

			FDSNStationXML root = (FDSNStationXML) unmarshaller
					.unmarshal(inputStream);
			if (root == null) {
				// TODO: throw not found exception
			}
			return root.getNetwork();

		} catch (JAXBException e) {
			IOException ioe = new IOException(e.getMessage());
			ioe.setStackTrace(e.getStackTrace());
			throw ioe;
		}

	}

	public List<Network> fetch(Criteria criteria, OutputLevel level)
			throws NoDataFoundException, CriteriaException, IOException,
			ServiceNotSupportedException {
		if (logger.isLoggable(Level.FINER)) {
			logger.entering(this.getClass().getName(), "fetch(criteria, url)",
					new Object[] { criteria, level });
		}

		StringBuilder paramsString = new StringBuilder(criteria.toUrlParams()
				.get(0));

		if (level == OutputLevel.NETWORK) {
			paramsString.append("&level=network").toString();
		} else if (level == OutputLevel.STATION) {
			paramsString.append("&level=station").toString();
		} else if (level == OutputLevel.CHANNEL) {
			paramsString.append("&level=channel").toString();
		} else if (level == OutputLevel.RESPONSE) {
			paramsString.append("&level=response").toString();
		} else {
			paramsString.append("&level=station").toString();
		}
		return this.fetch(this.baseUrl + "query?" + paramsString.toString());

	}

	/**
	 * Find stations based on the criteria provided. This will download data
	 * from IRIS station service (http://www.iris.edu/ws/station). The default
	 * level is station
	 * 
	 * @param criteria
	 * @param level
	 * @return List of all stations meeting the criteria provided.
	 * @throws NoDataFoundException
	 * @throws CriteriaException
	 * @throws ServiceNotSupportedException
	 */
	/*
	 * public List<Network> fetch(Criteria criteria, OutputLevel level) throws
	 * NoDataFoundException, CriteriaException, IOException,
	 * ServiceNotSupportedException {
	 */

	public List<Network> fetch(String url) throws NoDataFoundException,
			CriteriaException, IOException, ServiceNotSupportedException {
		if (logger.isLoggable(Level.FINER)) {
			logger.entering(this.getClass().getName(), "fetch(String url)",
					new Object[] { url });
		}

		URL theQuery = null;
		try {
			theQuery = new URL(url);

		} catch (MalformedURLException e) {
			throw new IOException("Invalid url: " + url);
		}

		HttpURLConnection connection = null;

		try {
			connection = getConnection(theQuery);
			connection.setRequestMethod("GET");

			String uAgent = this.userAgent;
			if (this.getAppName() != null && !"".equals(this.appName)) {
				uAgent = uAgent + " (" + this.appName + ")";
			}
			connection.setRequestProperty("User-Agent", uAgent);

			connection.setRequestProperty("Accept", "application/xml");
			connection.setRequestProperty("Accept-Encoding", "gzip, deflate");
			connection.connect();

			int responseCode = connection.getResponseCode();
			InputStream inputStream = responseCode != HTTP_OK ? connection
					.getErrorStream() : connection.getInputStream();
			if ("gzip".equals(connection.getContentEncoding())) {
				inputStream = new GZIPInputStream(inputStream);
			}
			switch (responseCode) {
			case 404:
				if (logger.isLoggable(WARNING))
					logger.warning("No data Found for the GET request "
							+ theQuery + StringUtil.toString(inputStream));
				return null;
			case 204:
				if (logger.isLoggable(WARNING))
					logger.warning("No data Found for the GET request "
							+ theQuery);
				throw new NoDataFoundException("No data found for: " + theQuery);
			case 400:
				if (logger.isLoggable(SEVERE))
					logger.severe("An error occurred while making a GET request "
							+ theQuery + StringUtil.toString(inputStream));
				throw new CriteriaException("Bad request parameter: "
						+ StringUtil.toString(inputStream));
			case 500:
				if (logger.isLoggable(WARNING))
					logger.severe("An error occurred while making a GET request "
							+ theQuery + StringUtil.toString(inputStream));
				throw new IOException(StringUtil.toString(inputStream));
			case 200:
				XMLInputFactory xmlFactory = XMLInputFactory.newInstance();

				XMLEventReader xmlEventReader = xmlFactory
						.createXMLEventReader(inputStream);

				EventFilter filter = new EventFilter() {
					public boolean accept(XMLEvent event) {
						return event.isStartElement();
					}
				};

				JAXBContext jc = JAXBContext
						.newInstance(edu.iris.dmc.fdsn.station.model.ObjectFactory.class);
				Unmarshaller unmarshaller = jc.createUnmarshaller();

				XMLEventReader xmlEventFilteredReader = xmlFactory
						.createFilteredReader(xmlEventReader, filter);

				StartElement se = (StartElement) xmlEventFilteredReader
						.nextEvent();

				if (!"Network".equals(se.getName().getLocalPart())) {
					// TODO: throw exception
				}

				List<edu.iris.dmc.fdsn.station.model.Network> networks = new ArrayList<edu.iris.dmc.fdsn.station.model.Network>();
				XMLEvent e = null;
				Network network = null;

				while ((e = xmlEventFilteredReader.peek()) != null) {
					if (e.isStartElement()
							&& (se = ((StartElement) e)).getName()
									.getLocalPart().equals("Network")) {

						network = new Network();
						networks.add(network);
						Iterator<Attribute> attributes = se.getAttributes();
						while (attributes.hasNext()) {
							Attribute attribute = attributes.next();
							String attributeName = attribute.getName()
									.getLocalPart();
							if (attributeName.equals("code")) {
								network.setCode(attribute.getValue());
							} else if (attributeName.equals("startDate")) {
								String startString = attribute.getValue();
								if (startString != null) {
									SimpleDateFormat sdf = new SimpleDateFormat(
											"yyyy-MM-dd'T'HH:mm:ss");
									try {
										Date date = sdf.parse(startString);
										GregorianCalendar c = new GregorianCalendar();
										c.setTime(date);
										XMLGregorianCalendar cal2 = DatatypeFactory
												.newInstance()
												.newXMLGregorianCalendar(c);
										network.setStartDate(cal2);
									} catch (ParseException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									} catch (DatatypeConfigurationException e2) {
										// TODO Auto-generated catch block
										e2.printStackTrace();
									}
								}
							} else if (attributeName.equals("endDate")) {
								String endString = attribute.getValue();
								if (endString != null) {
									SimpleDateFormat sdf = new SimpleDateFormat(
											"yyyy-MM-dd'T'HH:mm:ss");
									try {
										Date date = sdf.parse(endString);
										GregorianCalendar c = new GregorianCalendar();
										c.setTime(date);
										XMLGregorianCalendar cal2 = DatatypeFactory
												.newInstance()
												.newXMLGregorianCalendar(c);
										network.setEndDate(cal2);
									} catch (ParseException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									} catch (DatatypeConfigurationException e2) {
										// TODO Auto-generated catch block
										e2.printStackTrace();
									}
								}
							} else if (attributeName.equals("restrictedStatus")) {
								network.setRestrictedStatus(RestrictedStatusType
										.fromValue(attribute.getValue()));
							} else {
								// Do nothing for now
							}
						}
						xmlEventFilteredReader.next();
					} else if (e.isStartElement()
							&& (se = ((StartElement) e)).getName()
									.getLocalPart().equals("Description")) {
						XMLEvent event = xmlEventFilteredReader.nextEvent();
						if (event.isCharacters()) {
							network.setDescription(event.asCharacters()
									.getData());
						}
					} else if (e.isStartElement()
							&& (se = ((StartElement) e)).getName()
									.getLocalPart()
									.equals("TotalNumberStations")) {
						XMLEvent event = xmlEventFilteredReader.nextEvent();
						if (event.isCharacters()) {
							String data = event.asCharacters().getData();
							if (data != null) {
								network.setTotalNumberStations(new BigInteger(
										data));
							}
						}
					} else if (e.isStartElement()
							&& (se = ((StartElement) e)).getName()
									.getLocalPart().equals("Station")) {
						Object o = unmarshaller.unmarshal(xmlEventReader,
								edu.iris.dmc.fdsn.station.model.Station.class);

						if (o instanceof javax.xml.bind.JAXBElement) {
							Object jo = ((JAXBElement) o).getValue();
							if (jo instanceof edu.iris.dmc.fdsn.station.model.Station) {
								edu.iris.dmc.fdsn.station.model.Station station = (edu.iris.dmc.fdsn.station.model.Station) jo;
								network.addStation(station);
							}
						}
					} else {
						xmlEventFilteredReader.next();
					}
					// xmlEventFilteredReader.next();

				}

				if ("text/plain".equals(connection.getContentType())) {
					System.out.println("Size: " + networks.size());
				}
				connection.disconnect();

				if (logger.isLoggable(Level.FINER)) {
					logger.exiting(this.getClass().getName(),
							"fetch(EventCriteria criteria)");
				}

				return networks;
			default:
				throw new IOException(connection.getResponseMessage());
			}

		} catch (JAXBException e) {
			throw new IOException(e);
		} catch (XMLStreamException e) {
			e.printStackTrace();
			throw new IOException(e);
		} finally {
			if (connection != null) {
				connection.disconnect();
			}
		}

	}
}
